=head
	包    名: DataCopperArea.pm
	编码人员: 洪柏青
	功    能: 计算铜面积模块
	编码时间: 2022-11-11
=cut

package DataCopperAreaUp;

use strict;
use warnings;

use utf8;


#---计算外层铜面积
sub Data_Calc_outer_area{
	my $self    = shift;
	my $step    = shift;
	my $thk     = shift;

	my $i       = 1;
    my $matrix  = $self->{matrix};

	foreach my $layer (sort {$matrix->{$a}{row} <=> $matrix->{$b}{row}} keys %{$matrix}) {
		if ($matrix->{$layer}{context} =~ /board/i && $matrix->{$layer}{layer_type} =~ /signal/i && $matrix->{$layer}{side} =~ /top|bot/i) {
			if ($matrix->{$layer}{side} =~ /top/i) {
			   my ($total,$percent) = $self->Di_outer_copper_area($layer,$thk,"first");
			   $self->{area}{$step}{$layer}{total}   = $total;
	           $self->{area}{$step}{$layer}{percent} = $percent;
			   $self->{area}{$step}{$layer}{number}  = $i;
			}elsif ($matrix->{$layer}{side} =~ /bot/i) {
			   my ($total,$percent) = $self->Di_outer_copper_area($layer,$thk,"second");	
			   $self->{area}{$step}{$layer}{total}   = $total;
	           $self->{area}{$step}{$layer}{percent} = $percent;
			   $self->{area}{$step}{$layer}{number}  = $i;  
			}
			#$self->PAUSE("===$layer");
            $i++
		}
	}
}

#---计算内层铜面积
sub Data_Calc_inner_area{
	my $self    = shift;
	my $step    = shift;

	my $i       = 1;
    my $matrix  = $self->{matrix};
 
	#--计算内层铜面积
	foreach my $layer (sort {$matrix->{$a}{row} <=> $matrix->{$b}{row}} keys %{$matrix}) {
		if ($matrix->{$layer}{context} eq "board" && $matrix->{$layer}{side} =~ /inner/i) {
			$self->work_layer($layer); 
			my ($total,$percent) = $self->Di_outer_copper_area($layer,0,"first");

			$self->{area}{$step}{$layer}{total}   = $total;
	        $self->{area}{$step}{$layer}{percent} = $percent;
			$self->{area}{$step}{$layer}{number}  = $i; 
			$i++
		}
	}
}

#--计算金面积(绿油在铜面上的面积，要包含孔铜厚)#----沉金面积
sub Data_calc_soldermask_area{
	my $self    = shift;
	my $step    = shift;
	my $thk     = shift;

	my $matrix  = $self->{matrix};
	my $top_signal_Lay = "";#--顶层线路
	my $bot_signal_Lay = "";#--底层线路

    #---获取外层线路层层名称
    foreach my $layer (sort {$matrix->{$a}{row} <=> $matrix->{$b}{row}} keys %{$matrix}) {
		if ($matrix->{$layer}{context} =~ /board/i && $matrix->{$layer}{layer_type} =~ /signal/i && $matrix->{$layer}{side} =~ /top|bot/i) {
			if ($matrix->{$layer}{side} =~ /top/) {
				$top_signal_Lay = $layer;
			}
			elsif ($matrix->{$layer}{side} =~ /top/) {
				$bot_signal_Lay = $layer;
			}
		}
	}

    #---开始计算
	my $i       = 1;
	foreach my $layer (sort {$matrix->{$a}{row} <=> $matrix->{$b}{row}} keys %{$matrix}) {
		if ($matrix->{$layer}{context} =~ /board/i && $matrix->{$layer}{layer_type} =~ /solder_mask/i && $matrix->{$layer}{side} =~ /top|bot/i) {
			if ($matrix->{$layer}{side} =~ /top/i) {	
			   my ($total,$percent) = $self->Di_outer_exposed_area($top_signal_Lay,$thk,$layer,"first");
			   $self->{area}{$step}{$layer}{total}   = $total;
	           $self->{area}{$step}{$layer}{percent} = $percent;
			   $self->{area}{$step}{$layer}{number}  = $i;
			}elsif ($matrix->{$layer}{side} =~ /bot/i) {
			   my ($total,$percent) = $self->Di_outer_exposed_area($bot_signal_Lay,$thk,$layer,"second");	
			   $self->{area}{$step}{$layer}{total}   = $total;
	           $self->{area}{$step}{$layer}{percent} = $percent;
			   $self->{area}{$step}{$layer}{number}  = $i; 
			}
            $i++
		}
	}

    $self->COM("editor_page_close");
}

#--上传每层铜面积到 MES
sub UpLoadCopperAreaToMes{
	my $self    = shift;
	my $wkStep  = shift;


	my $job     = $self->{Job};

	my $matrix  = $self->{matrix};
    
	#$self->get_qingda_connect();
	$self->get_MSSQL_connect('192.168.0.58,8588','DATABASE4QD','EMS4QD','EMSPW4QD+2016');
	if ($self->{MsSql}{dbh} !~ /dbi/i) {
		print "no conect database\n";
		$self->pauseMessage("提醒","上传电脑面积时连接 MES 数据库失败!",300,300);
		return;
	}else{
		print "Conect ok\n";
	}

	#--删除旧的记录
	$self->sqlExecuteMsSql("delete from T_GS_ResidualCopper where Pdctno_ = \'$job\'");

    #insert into T_GS_OutInfoList(Pdctno_,Techno_,Itemno_,Itemname_,Itempara_,Displayord_,Keyin_,Indate_) values('HDtest001','466','0007','面积','55',0,'cam32',getdate()); 

	# id_,Pdctno,LineID_,LayerName_,Itempara_,ItemparaPers_,Keyin_,Indate_
	#my $sql = "insert into T_GS_ResidualCopper(id_,Pdctno,LineID_,LayerName_, Itempara_,ItemparaPers_,Keyin_,Indate_) values(null,?,?,?, ?,?,?,?)";

	
    my $Keyin = $self->{hostname};
    
	my $upDataInfo = "";
    my $compFlag = "ok";
    my $i        = 1;
	foreach my $layer (sort {$matrix->{$a}{row} <=> $matrix->{$b}{row}} keys %{$matrix}) {
		
		if ( ($matrix->{$layer}{context} =~ /board/i && $matrix->{$layer}{layer_type} =~ /signal/i && $matrix->{$layer}{side} =~ /top|bot/i) ||
			 ($matrix->{$layer}{context} eq "board" && $matrix->{$layer}{side} =~ /inner/i) ){
			my @areaLines;

			my $total   = $self->{area}{$wkStep}{$layer}{total};
			my $percent = $self->{area}{$wkStep}{$layer}{percent};

			#push @areaLines,$job,$i,$layer,$total,$percent,$Keyin,"getdate\(\)";
			#$upDataInfo .= "@areaLines\n";
            
			my $insertSql = "insert into T_GS_ResidualCopper(Pdctno_,LineID_,LayerName_, Itempara_,ItemparaPers_,Keyin_,Indate_) 
			                 values(\'$job\',$i,\'$layer\',$total,$percent,\'$Keyin\',getdate\(\))";

			#---插入数据
			$self->sqlExecuteMsSql($insertSql);
			#my $status = $self->insert_SQL_table($sql,@areaLines);
		    #if ($status =~ /ng/i) {
			#    $compFlag = "ng";
		    #}
		}
	}
	print "$upDataInfo\n";
    
	#--断开连接
	$self->disconnectMsSql();

	if ($compFlag !~ /ok/i) {
		$self->pauseMessage("提醒","电脑面积写入到 MES 数据库失败!",300,300);
	}
}

#---获取面积数据并上传--输出的残铜率
sub Data_calc_area_main{
	my $self    = shift;
    my $wkStep  = "pnl";
	if ($self->{Step} =~ /panel|pnl/) {
		$wkStep  = $self->{Step};
	}
    
	#---清空数据
	$self->{area} = {};

	$self->open_step($wkStep,"inch");
    
	#--计算内层面积
	$self->Data_Calc_inner_area($wkStep);

	#--计算外层面积
	$self->Data_Calc_outer_area($wkStep,$self->{mesinfo}{board_thk} * 25.4);

	#--上传内，外层线路面积到 MES
	$self->UpLoadCopperAreaToMes($wkStep,);
}

1;

