=head
	包    名: DataDrillRoutUp.pm
	编码人员: 洪柏青
	功    能: 获取钻孔,锣带信息，并上传
	编码时间: 2022-11-11
=cut

package DataDrillRoutUp;

use strict;
use warnings;

use utf8;


#--获取单元内最小的孔
sub erp_get_small_unit_hole{
	my $self      = shift;
	my $edit_step = shift;
	my $allholes = $self->Di_get_allHoles($edit_step,"drl","mm");
	if ($#{$allholes} >= 0) {
		$self->addAttrs("output_param","mindrlSizeMM" => sprintf("%.3f",$allholes->[0] / 1000));
	}
}

#----获取电镀纵横比	沉铜	电镀纵横比  板厚除以最小孔径
sub erp_get_thk_scale_hole_plate_scale{
	my $self = shift;
	my $list = $self->{matrix};
	if (! exists $list->{"drl"}) {
		return;
	}
	my $fealist = $self->SYMS_HIST_LIST("edit","drl","mm");
	my @symbol;
	foreach my $key (%$fealist) {
		if ($fealist->{$key}{symbol} =~ /r(\S+)/) {
			push @symbol,$1 / 1000;
		}
	}
	@symbol = sort {$a <=> $b} @symbol;
	#print "symbol: @symbol\n";

	$self->{Di}{plate_scale} = sprintf("%.2f",$self->{item}{board_thk} / $symbol[0]);
	$self->{Di}{plate_scale} .= ':1';
}

#---获取锣程
sub erp_get_calcRoutLength{
	my $self     = shift;
	my $wkStep   = shift;
	my $list     = $self->{matrix};
	if (! exists $list->{"rou"}) {
		return;
	}
	my $layer    = "rou";
    my $destLay  = $layer."-path$$";

    $self->open_step($wkStep);

    $self->COM("units,type=mm");
	$self->COM("compensate_layer,source_layer=$layer,dest_layer=$destLay,dest_layer_type=document");

    $self->work_layer($destLay);

	$self->COM("sel_change_sym,symbol=r10,reset_angle=no");
    my $STATUS = $self->{STATUS};
	if ($STATUS != 0) {
		$self->pauseMessage("锣程计算","当前层有特殊 symbol,不能计算",500,300);
		return;
	}
    $self->COM("copper_area,layer1=$destLay,layer2=,drills=no,resolution_value=1,x_boxes=3,y_boxes=3,area=no,dist_map=yes");
	my @data      = split /\s+/,$self->{COMANS};
	#--print "@data\n";
	my $area      = $data[0] / 0.01;#--- mm
	my $unit_m    = sprintf("%.3f",$area / 1000);         #--- m (米)
	my $unit_inch = sprintf("%.3f",$area / 0.0254 / 1000);#--- inch(英寸)
	my $unit_mil  = sprintf("%.3f",$area / 0.0254);       #--- mil(毫英寸)

	#--- foot(英尺) 1英寸 = 0.083333英尺 = 1/12  1 foot 英尺=12 inches 英寸
	my $unit_foot = sprintf("%.3f",$area / 0.0254 / 1000 / 12);   

   	#$self->delete_layer($destLay);

	#print "$area  ---> mm   $unit_m\n";
	#print "$area  ---> inch $unit_inch\n";
	#print "$area  ---> mil  $unit_mil\n";
	#print "$area  ---> foot $unit_foot\n";
	$self->{Di}{routPath} = $area;
	my $text  =  $area.' mm(毫米)'."\n";
	   $text .=  $unit_m.' m(米)'."\n";
	   $text .=  $unit_inch.' inch(英寸)'."\n";
	   $text .=  $unit_foot.' foot(英尺)'."\n";
    $self->COM("units,type=inch");
    #$self->pauseMessage("锣程计算","$text",500,300);
}


#####**********钻孔流程表，输出钻孔要更新到此,   
sub UploadDrill_Get_GuideMain{
	my $self     = shift;
	my $job      = shift;
	my $layerNum = shift;

	#--获取 Guid_ 从表名称: T_ppe_MakeTechDrill4Info  #---上传钻带第一步查询，是否存在，, 这里一定要加上条件 型号
	my $sqlGetGuid4DrillMain  = "select Guid_ from T_ppe_MakeTechDrill4Info where Pdctno4Drill_ = \'$job\'";
	my $guidMain = $self->inquire_qian_data_sql($sqlGetGuid4DrillMain);
	print "Sql:$sqlGetGuid4DrillMain==>guidMain: $guidMain\n";

	if ($guidMain eq '') {
		#--如没有，需要插入到 表名称:T_ppe_MakeTechDrill4Info 表。
		my $insertSql  = "insert T_ppe_MakeTechDrill4Info(Guid_,Pdctno4Drill_,AreaID4Drill_,DrillLayerNum_,InDate) values(NEWID(),\'$job\',15,$layerNum,getdate())";
        $self->sqlExecuteMsSql($insertSql);

		#--获取 Guid_ 
		$guidMain = $self->inquire_qian_data_sql($sqlGetGuid4DrillMain);
        print "Sql2:$sqlGetGuid4DrillMain==>guidMain2: $guidMain\n";

		#--需要将 guidMain 更新到 型号总表:表名称:T_ppe_MakeModuleNo, 这里一定要加上条件 型号
		if ($guidMain ne '') {
			my $updateSql = "update T_ppe_MakeModuleNo set Guid4Drl_ = \'$guidMain\' where PdctNo_ = \'$job\'";
			$self->sqlExecuteMsSql($updateSql);
		}
	}

	return $guidMain;
}


#####**********插入钻带汇总表1  表名称:T_ppe_MakeTechDrill4PNLParameter4GS
sub UploadDrill_insert_Drill4PNLParameter4GS{
	my $self       = shift;
	my $guidMain   = shift;
	my $drlLayName = shift;#--钻带名称
	my $firstLay   = shift;#--开始层
	my $endLay     = shift;#--结束层


    #--获取之前插入的 id 号
	my $selectOLDSql  = "select Guid_ from T_ppe_MakeTechDrill4PNLParameter4GS  where Guid4DrillMain_ = \'$guidMain\' and DrillName_ = \'$drlLayName\'";
	my $dataOLD       = $self->FetchrowArrayMsSql($selectOLDSql);
  	if ($#{$dataOLD} >= 0) {
		#$self->PAUSE("===$#{$dataOLD}");
		my $Guid4Drill =$dataOLD->[0][0];

		#$self->PAUSE("DELETE===$Guid4Drill");

		#--删除子表旧的记录
	    $self->sqlExecuteMsSql("delete from T_ppe_MakeTechDrill4PNLToolDiameter4GS  where Guid4Drill_ = \'$Guid4Drill\'");

		#--删除旧的记录, 表名称:T_ppe_MakeTechDrill4PNLParameter4GS 这里要带条件  Guid4DrillMain_ 和 DrillName_  
	    my $deletesql  = "delete from T_ppe_MakeTechDrill4PNLParameter4GS  where Guid4DrillMain_ = \'$guidMain\' and DrillName_ = \'$drlLayName\'";
	    $self->sqlExecuteMsSql($deletesql);
	}
	#$self->PAUSE("END===$#{$dataOLD}");


	my $Keyin = $self->{hostname};
	#--插入层到表 表名称:T_ppe_MakeTechDrill4PNLParameter4GS
	#my $insertSql = "insert into T_ppe_MakeTechDrill4PNLParameter4GS(Guid_,Guid4DrillMain_,DrillName_,InUserNo_,Indate_,BeginLyr_,EndLyr_)
	#values(?,?,?,?, ?,?,?)";
	my $insertSql = "insert into T_ppe_MakeTechDrill4PNLParameter4GS(Guid_,Guid4DrillMain_,DrillName_,InUserNo_,Indate_,BeginLyr_,EndLyr_)
	values(NEWID\(\),\'$guidMain\',\'$drlLayName\',\'$Keyin\',getdate\(\),$firstLay,$endLay)";

   $self->sqlExecuteMsSql($insertSql);

    
	#--获取插入的 id 号
	my $selectSql  = "select Guid_ from T_ppe_MakeTechDrill4PNLParameter4GS  where Guid4DrillMain_ = \'$guidMain\' and DrillName_ = \'$drlLayName\'";
	my $data       = $self->FetchrowArrayMsSql($selectSql);
  	if ($#{$data} >= 0) {
		print "T_ppe_MakeTechDrill4PNLParameter4GS guidMain:$guidMain Guid_ ==$data->[0][0]\n";
	    #$self->PAUSE("====guid4Drill:$data->[0][0]");
		return $data->[0][0];
	}
	return "";
}

#---钻带的明细-插入钻孔层 到 T_ppe_MakeTechDrill4PNLToolDiameter4GS
sub UploadDrill_insert_ToolDrill{
	my $self          = shift;
	my $Guid4Drill    = shift;
	my $drillToolList = shift;


	#--插入前删除旧的记录
	my $deletesql  = "delete from T_ppe_MakeTechDrill4PNLToolDiameter4GS  where Guid4Drill_ = \'$Guid4Drill\'";
	$self->sqlExecuteMsSql($deletesql);

	#--插入层到新记录
	#--toolShape:  0 Hole  1 锣刀 2 Slot  3 毛刺   4 控深 5 背钻 6 邮票孔 7 清角孔 8 辅助孔 9 辅助槽
	#--10 防呆孔 11 工具孔 12 除尘孔

	#--Toolkind  种类的意思:
	#--1 Via  2 PTH  3 NPTH  4 BGA  5 PressFit
    #--Guid_自动生成ID,Guid4Drill_钻带汇总表ID,Toolno刀序,toolShape用途,
	#toolkind种类,tooldia刀径,storedia成品孔径,blength槽长,
	#qty数量,posittol正公差,minustol负公差,Captions备注

	for my $i (0..$#{$drillToolList}) {
		#$lineTxt .= "finishSize=$drillToolList->[$i][0];type=$drillToolList->[$i][1];type2=$drillToolList->[$i][8];
		#tolz=$drillToolList->[$i][2];tolf=$drillToolList->[$i][3];count=$drillToolList->[$i][6];
		#remark=$drillToolList->[$i][9];tnum=$drillToolList->[$i][10];drillSize=$drillToolList->[$i][12];
		#finishSlot=$drillToolList->[$i][14];drillSlot=$drillToolList->[$i][15];slotHit=$drillToolList->[$i][7]\n";
		my $toolKind = "";
		if ($drillToolList->[$i][1] =~ /Y/i) {
			$toolKind = 2;
		}elsif ($drillToolList->[$i][1] =~ /N/i) {
			$toolKind = 3;
		}
		my $blength = 0;
		if ($drillToolList->[$i][15] =~ /\d/) {
			$blength = $drillToolList->[$i][15];
		}
		my $insertSql = "insert into T_ppe_MakeTechDrill4PNLToolDiameter4GS(
		  Guid_,Guid4Drill_,toolno,toolShape,
		  toolkind,tooldia,storedia,blength,
		  qty,posittol,minustol,Captions)
	      values(NEWID\(\),\'$Guid4Drill\',\'$drillToolList->[$i][10]\',\'11\',
			     \'$toolKind\',$drillToolList->[$i][12],$drillToolList->[$i][0],$blength, 
			     $drillToolList->[$i][6],$drillToolList->[$i][2],$drillToolList->[$i][3],\'$drillToolList->[$i][9]\')";

	    #---插入数据
		print "$insertSql\n";
	    $self->sqlExecuteMsSql($insertSql);
	} 

	#--Guid_自动生成ID,Guid4Drill_钻带汇总表ID,Toolno刀序,toolShape用途,toolkind种类,tooldia刀径,storedia成品孔径,blength槽长,qty数量,posittol正公差,minustol负公差,Captions备注
	#my $insertSql = "insert into T_ppe_MakeTechDrill4PNLToolDiameter4GS(Guid_,Guid4Drill_,toolno,toolShape,toolkind,tooldia,storedia,blength,qty,posittol,minustol,Captions)
	#values(NEWID\(\),\'$Guid4Drill\',\'T01\',\'11\',    \'\',0.503,0,0,  3,0,0,\'测试孔\')";

	#---插入数据
	#$self->sqlExecuteMsSql($insertSql);
}

sub UpLoadDrillInfoToMes{
	my $self          = shift;
	my $boardLayer    = shift;#--板的层数
	my $drlLayName    = shift;#--钻孔层名称
	my $drlFirstLay   = shift;#--钻带开始层
	my $drlEndLay     = shift;#--钻带结束层 
	my $minDrill      = shift;#--最小孔经
	my $drillToolList = shift;#--钻孔列表

	my $job     = uc($self->{Job});

	my $matrix  = $self->{matrix};
    
	$self->get_MSSQL_connect('192.168.0.58,8588','DATABASE4QD','EMS4QD','EMSPW4QD+2016');
	if ($self->{MsSql}{dbh} !~ /dbi/i) {
		print "no conect database\n";
		$self->pauseMessage("提醒","上传 钻孔 时连接 MES 数据库失败!",300,300);
		return;
	}else{
		print "Conect ok\n";
		#$self->pauseMessage("数据库连接OK");
		#$self->message_ok("数据库操作失败\n $@") if($@);
	}
    
    #$self->PAUSE("boardLayer:$boardLayer =drlLayName:$drlLayName minDrill:$minDrill ==>drlFirstLay:$drlFirstLay ,drlEndLay:$drlEndLay");
    
	#--钻孔上传需要设定的参数
	#my $boardLayer  = 4;    #--板的层数
    #my $drlLayName  = "drl";#--钻孔层名称
	#my $drlFirstLay = 1;    #--钻带开始层
	#my $drlEndLay   = 8;    #--钻带结束层 
	#my $minDrill    = 0.4;  #--最小孔经 
	my $mincuhole   = 0;    #--最小孔到铜
	
	$self->{hostname}    = `hostname`;#--获取主机名称
    chomp $self->{hostname};#--去掉回车


	#--获取主表id 表名称:Guid4DrillMain_,   传入2 个参数:	
	my $guidMain = $self->UploadDrill_Get_GuideMain($job,$boardLayer);#--型号名称，层数
    print "guidMain: $guidMain\n";
	if ($guidMain eq "") {
		$self->pauseMessage("提醒","获取钻孔型号总表guide_ 失败!",300,300);
	}

	#--插入钻带汇总表1  表名称:T_ppe_MakeTechDrill4PNLParameter4GS   传入4 个参数:	
	my $guid4Drill = $self->UploadDrill_insert_Drill4PNLParameter4GS($guidMain,$drlLayName,$drlFirstLay,$drlEndLay);#--  主表的ID,钻带名称,钻带开始层,钻带结束层  
	if ($guid4Drill eq "") {
		$self->pauseMessage("提醒","插入钻带汇总表1  T_ppe_MakeTechDrill4PNLParameter4GS 失败!",300,300);
	}

	#--插入钻带汇总表2  表名称:T_ppe_MakeTechDrillParameter   传入 7 个参数:	
	$self->UploadDrill_insert_MakeTechDrillParameter($job,$guidMain,$guid4Drill,$drlLayName,$boardLayer,$drlFirstLay,$drlEndLay,$mincuhole,$minDrill);#--型号名称，主表的ID,钻带汇总表1ID,钻带名称,钻带开始层,钻带结束层,最小孔到铜,最小孔经  

    #---插入钻刀刀表名称为:  表名称:T_ppe_MakeTechDrill4PNLToolDiameter4GS
     $self->UploadDrill_insert_ToolDrill($guid4Drill,$drillToolList);#--钻带汇总表1ID


	#--断开连接
	$self->disconnectMsSql();
}

#####**********插入钻带汇总表2  表名称:T_ppe_MakeTechDrillParameter   传入 7 个参数:
sub UploadDrill_insert_MakeTechDrillParameter{
	my $self       = shift;
	my $job        = shift;#--型号名称
	my $guidMain   = shift;#--主表的ID
	my $guid4Drill = shift;#--带汇总表1ID
	my $drlLayName = shift;#--钻带名称
	my $boardLayer = shift;#--板的层数
	my $firstLay   = shift;#--开始层
	my $endLay     = shift;#--结束层
	my $mincuhole  = shift;#--最小孔到铜
	my $minDrill   = shift;#--最小孔经 
	

	
	#--删除旧的记录, 表名称:T_ppe_MakeTechDrillParameter 这里要带条件  Guid4DrillMain_ 和 DrillName_
	my $deletesql  = "delete from T_ppe_MakeTechDrillParameter  where Guid4DrillMain_ = \'$guidMain\' and DrillName_ = \'$drlLayName\'";
	$self->sqlExecuteMsSql($deletesql);

    #my $Keyin     = $self->{hostname};
	my $selectSql = "select MI_MakeBeginUserNo_ from V_WF4Order4MI where Pdct_NAME_ + REV_ = \'$job\'";
    my $Keyin     = $self->inquire_qian_data_sql($selectSql);

	#--插入层到表 表名称:T_ppe_MakeTechDrillParameter
	#my $insertSql = "insert into T_ppe_MakeTechDrillParameter(Guid_,Guid4DrillMain_,DrillName_,InUserNo_,Indate_,BeginLyr_,EndLyr_,MinCuHole_,MinDrill_)
	#values(\'$guid4Drill\',\'$guidMain\',\'$drlLayName\',\'$Keyin\',getdate\(\),$firstLay,$endLay,$mincuhole,$minDrill)";

	#--插入层到表 表名称:T_ppe_MakeTechDrillParameter
	my $insertSql = "insert into T_ppe_MakeTechDrillParameter(Guid_,Guid4DrillMain_,DrillName_,DrillFileName_,InUserNo_,Indate_,MaxLyr_,BeginLyr_,EndLyr_,MinDrill_)
	values(\'$guid4Drill\',\'$guidMain\',\'$drlLayName\',\'0\',\'$Keyin\',getdate\(\),$boardLayer,$firstLay,$endLay,$minDrill)";

	$self->sqlExecuteMsSql($insertSql);

	#print "$insertSql\n";

	#$self->PAUSE("====guid4Drill:$guid4Drill guidMain:$guidMain");

	#my @drlLayer;
	#push @drlLayer,"\'$guid4Drill\'","\'$guidMain\'","\'$drlLayName\'","\'$Keyin\'","getdate\(\)",$firstLay,$endLay,$mincuhole,$minDrill;
	#my $upDataInfo = "@drlLayer\n";
	#my $status = $self->insert_SQL_table($insertSql,@drlLayer);
    
}

1;

