#!/usr/bin/perl

=head
     01/29/16 HongBaiQing
=cut

package OutputBase;

use strict;
use warnings;
use Encode;
use File::Basename;
use File::Copy;
use File::Path qw(make_path);
use utf8;
use Time::localtime;
use Data::Dumper;
#============================
#---空层 
sub empty_layer{
	my $self = shift;
	my ($step,$layer,$break) = @_;
	my $tmpFile;
	if (defined $break) {
		$tmpFile  = $self->FEATURES_break_sr($step,$layer);			
	}
	else{
		$tmpFile  = $self->FEATURES($step,$layer);
	}					
	my $features = $self->ReadFile($tmpFile);
	unlink $tmpFile;


	if (@{$features} <2 ) {
		return 0;
	}
	else{
		my $count = $#{$features};
		return $count;
	}
}
#=====================================================

#--计算层数据
sub get_layers_count{
	my $self   = shift;
    $self->{lay_count}{signal}      = 0;
	$self->{lay_count}{solder_mask} = 0;
	$self->{lay_count}{silk_screen} = 0;
	my $matrix = $self->{matrix};


	foreach my $layer (sort {$matrix->{$a}{row} <=> $matrix->{$b}{row}} keys %$matrix) {
        if ($matrix->{$layer}{layer_type} =~ /signal|power_ground|mixed/i && $matrix->{$layer}{side} =~ /top|bot|inner/i) {
             $self->{lay_count}{signal} ++;
		}elsif ($matrix->{$layer}{layer_type} =~ /solder_mask/i && $matrix->{$layer}{side} =~ /top|bot/i) {
             $self->{lay_count}{solder_mask} ++;
		}elsif($matrix->{$layer}{layer_type} =~ /silk_screen/i  && $matrix->{$layer}{side} =~ /top|bot/i) {
             $self->{lay_count}{silk_screen} ++;
		}
	}
}

#===================================================
#===================================================
#$compare 要不要对比
#$report  要不要输出对比报告
#$self->out_Gerber_layer($type,$step,$layer,$outName,$xscale,$yscale,$nfone,$nftwo,$mir,$angle,"ss_ref_table/""","compare/null","report/null","reread/null","$pfix/null","$sfix/null");
sub out_Gerber_layer{
	my $self = shift;                                                                
	my ($type,$step,$layer,$outName,$xscale,$yscale,$nfone,$nftwo,$mir,$angle,$wheel,$compare,$report,$reread,$pfix,$sfix) = @_;
	my $tmpDir = "C:/tmp";

	my $prefix =  ($pfix =~ /null/) ? "" : $pfix ;
	my $suffix =  ($sfix =~ /null/) ? "_$$" : $sfix ;
	my $outLayer = "${prefix}${layer}$suffix";
	my $file = "$tmpDir/$outLayer";
	if (-e $file) {
		unlink $file ;
	}
	

		
	#--输出---
	$self->VOF;
	$self->COM("output_layer_reset");
	$self->COM("output_layer_set,layer=$layer,angle=$angle,mirror=$mir,x_scale=$xscale,y_scale=$yscale,\
			comp=0,polarity=positive,setupfile=,setupfiletmp=,line_units=inch,\
			gscl_file=,step_scale=no");
		

#	$self->COM("output,job=$self->{Job},step=$step,format=Gerber274x,dir_path=$tmpDir,\
#			prefix=$prefix,suffix=$suffix,break_sr=yes,break_symbols=no,break_arc=no,scale_mode=all,\
#			surface_mode=contour,min_brush=1,units=inch,coordinates=absolute,\
#			zeroes=none,nf1=$nfone,nf2=$nftwo,x_anchor=0,y_anchor=0,wheel=$wheel,x_offset=0,\
#			y_offset=0,line_units=inch,override_online=yes,film_size_cross_scan=0,\
#			film_size_along_scan=0,ds_model=RG6500");

	$self->COM("output,job=$self->{Job},step=$step,format=Gerber274x,dir_path=$tmpDir,\
			prefix=$prefix,suffix=$suffix,break_sr=yes,break_symbols=no,break_arc=no,scale_mode=nocontrol,\
			surface_mode=contour,min_brush=1,units=inch,coordinates=absolute,\
			zeroes=none,nf1=$nfone,nf2=$nftwo,x_anchor=0,y_anchor=0,wheel=$wheel,x_offset=0,\
			y_offset=0,line_units=inch,override_online=yes,film_size_cross_scan=0,\
			film_size_along_scan=0,ds_model=RG6500");


    my $status = $self->{STATUS};
	my $COMANS = $self->{COMANS};
	$self->VON;

	print "status:$status COMANS:$COMANS READANS:$self->{READANS}\n";

	if ($status == 66011) {
		#--SIP 错误可以输出，但是要提醒下
		$self->tipMessage("输出出问题","Step:$step Layer:$layer 输出有 SIP 错误",500,300);
	}
	elsif ($status != 0) {
		#if ($times =~ /first/i) {##这里传入的是第一次，脚本会尝试第二次输出
        $self->pauseMessage("输出出问题","Step:$step Layer:$layer 输出异常,可能是内存不足\n请将登陆 GENESIS 的黑屏窗口截图保存以便分析错误",500,300);                                
		return ;
	}
	

	$outName = encode('gb2312',$outName);
	my $dir = dirname $outName;
	make_path($dir);

	#--要对比，一定要读入
	if ($compare =~ /compare/) {
		$reread = "reread";
	}

	#----读入
	if ($reread =~ /reread/) {
		$self->COM("input_auto,path=$file,job=$self->{Job},step=$step");
		$status = $self->{STATUS};
		$COMANS = $self->{COMANS};
		if ($status != 0 || $COMANS != 0) {
			$self->pauseMessage("输出出问题","Step:$step Layer:$layer 文件读入失败",500,300);   
			unlink $file;
			return ;
		}
	}

	#---对比
	if ($compare =~ /compare/) {
		$self->work_affected_layer($outLayer);
		#--镜像回去
		if ($mir =~ /yes/i) {
			$self->COM("sel_transform,mode=anchor,oper=mirror,duplicate=no,x_anchor=0,
				y_anchor=0,angle=0,x_scale=1,y_scale=1,x_offset=0,y_offset=0");
		}


		#---涨缩值缩涨回去
		if ($xscale != 1 || $yscale != 1) {
			$xscale = 1/$xscale;
			$yscale = 1/$yscale;
			$self->COM("sel_transform,mode=anchor,oper=scale,duplicate=no,x_anchor=0,y_anchor=0,angle=0,
			         x_scale=$xscale,y_scale=$yscale,x_offset=0,y_offset=0");
			if ($self->{COMANS}) {
				$self->sel_contourize;
				$self->COM("sel_transform,mode=anchor,oper=scale,duplicate=no,x_anchor=0,y_anchor=0,angle=0,
			         x_scale=$xscale,y_scale=$yscale,x_offset=0,y_offset=0");
			}
		}

		#---不是0度旋转
		$angle =   ($angle == 90) ? 270 
				 : ($angle == 180) ? 180 
				: ($angle == 270) ? 90  : 0;	 
		 if ($angle) {
			 $self->COM("sel_transform,mode=anchor,oper=rotate,duplicate=no,x_anchor=0,
				y_anchor=0},angle=$angle,x_scale=1,y_scale=1,x_offset=0,y_offset=0");
		}



		my $map = "${layer}_comp";
		$self->COM("compare_layers,layer1=$outLayer,job2=$self->{Job},step2=$step,layer2=$layer,\
                layer2_ext=,tol=1,area=global,ignore_attr=,map_layer=$map,map_layer_res=200");
		my $diff_Com = $self->{COMANS};
		my $diff_sta = $self->{STATUS};
		#my $diff = 1;
		if($diff_Com == 0 && $diff_sta == 0) {
			move ($file,$outName) or die "$!";
			$self->delete_layer($outLayer);
			$self->delete_layer($map);			
		}
		else{
			unlink $file;
			#push @{$self->{error}},"output->${step}->${layer} 读入对比不一样";
			#--比对错误，比对结果层不要删除
			#$self->delete_layer($map);
			#$self->delete_layer($outLayer);
			$self->pauseMessage("输出出问题","Step:$step Layer:$layer  读入对比不一样",500,300);   
			print "output->${step}->${layer} diff-------------------\n";
			return ;
		}
		
		#---写入对比报告
		if ($report =~ /report/i) {	
			my $okFile = encode('gb2312',"output_compare_${layer}_ok.txt");
			$okFile = "$dir/$okFile";
			my $difFile = encode('gb2312',"output_compare_${layer}_dif.txt");
			$difFile = "$dir/$difFile";
			if (-e $difFile) {
				unlink $difFile;
			}
			if (-e $okFile) {
				unlink $okFile;
			}
			unless($diff_Com) {
				$self->WriteFile($okFile, "Compare layer: $layer     Compare result:ok\n");
			}
			else{
				$self->WriteFile($difFile, "Compare layer: $layer     Compare result:diff\n");
			}
		}
	}
	else{
		move ($file,$outName) or die "$!";
	}
	
}
#============================================

##################################################################################
##################################################################################

#=========================
#---测面积
sub measure_area{
	my $self = shift;
	my $step = shift;
	my @inners = ();
	my @outs = ();
	my @masks = (); 
	my $B_THK = $self->{jobUserInfo}{'板厚'} * 1000;   #这里是my单位
	$self->units("inch");

	delete $self->{matrix}{""};  #---不知在哪多了这层

	#---内层百分比
	my $info_file="c:/tmp/copper_$$";
	foreach my $layer (sort{$self->{matrix}{$a}{row}<=>$self->{matrix}{$b}{row}} keys %{$self->{matrix}} ) {
		if ($self->{matrix}{$layer}{context} =~ /board/i && $self->{matrix}{$layer}{side} =~ /inner/i) {
			$self->COM("copper_area, layer1=$layer,layer2=,drills=no,
				resolution_value=1,x_boxes=3,y_boxes=3,area=no,dist_map=yes,out_file=$info_file,out_layer=First");	

			my ($total,$percent) = ("","");

			open(MYFILE,"$info_file") or die "$!";
			while (<MYFILE>) {
				chomp;
				if (/Total copper.*:\s+(\S+)\s+sq\/in/i) {
					$total = $1;
					$total = sprintf ("%0.3f",$total/144);  #--平方英尺
				}elsif (/Percentage.*:\s+(\S+)/i) {
					$percent = $1;
				}
			}
			close (MYFILE);
			unlink $info_file;

			push @inners,[$layer,$total,$percent];
			$self->{area}{$step}{$layer}{total}   = $total;#--上传 MES 需要
	        $self->{area}{$step}{$layer}{percent} = $percent;#--上传 MES 需要
		}
    }



	#--外层面积 百分比
	foreach my $layer (sort{$self->{matrix}{$a}{row}<=>$self->{matrix}{$b}{row}} keys %{$self->{matrix}} ) {
		if ($self->{matrix}{$layer}{context} =~ /board/i && $self->{matrix}{$layer}{side} =~ /top|bot/i 
		&& $self->{matrix}{$layer}{layer_type} =~ /signal/i) {
			my $tmpFile ="c:/tmp/copper_$$";

			my ($layer1,$layer2,$outlayer) = ($layer,"","First");
			if ($self->{matrix}{$layer}{side} =~ /bot/i) {
				($layer1,$layer2,$outlayer) = ("",$layer,"Second");
			}
			my $halfThk = $B_THK/2  ;
			$halfThk = $halfThk/25.4;
			$self->COM("copper_area,layer1=$layer1,layer2=$layer2,drills=yes,consider_rout=no,drills_source=matrix,thickness=$halfThk,\
                resolution_value=1,drills_list=,x_boxes=3,y_boxes=3,area=no,dist_map=yes,out_layer=$outlayer,out_file=$tmpFile");

			my $STATUS = $self->{STATUS};
			if ($STATUS != 0) {
				$self->pauseMessage("跑板边计算面积","计算铜面积出错，可能是资料过大内存不足。请保存资料,关闭 GENESIS 重新跑板边");
			}
			my ($total,$percent) = ("","");
			open (LOGFILETMP, $tmpFile) or die("file not exist, $!");
			while (<LOGFILETMP>) {
				 chomp;
				if (/Total copper.*:\s+(\S+)\s+sq\/in/i) {
					$total = $1;
					$total = sprintf ("%0.3f",$total/144);  #--平方英尺
				}elsif (/Percentage.*:\s+(\S+)/i) {
					$percent = $1;
				}
			}
		   close (LOGFILETMP);
		   unlink $tmpFile;
		   push @outs,[$layer,$total,$percent];

			$self->{area}{$step}{$layer}{total}   = $total;#--上传 MES 需要
	        $self->{area}{$step}{$layer}{percent} = $percent;#--上传 MES 需要
	   }
	}

	#----外层沉金面积
	$self->units("mm");
	if (exists $self->{matrix}{tl} && exists $self->{matrix}{ts} && exists $self->{matrix}{bl} &&exists $self->{matrix}{bs}){
		$self->COM("exposed_area, layer1=tl,mask1=ts,layer2=bl,mask2=bs,
			mask_mode=or,drills=yes,consider_rout=no,ignore_pth_no_pad=no,drills_source=matrix,thickness=$B_THK,
			resolution_value=1,x_boxes=3,y_boxes=3,area=no,dist_map=yes,out_file=$info_file,out_layer=both");
		my $STATUS = $self->{STATUS};
		if ($STATUS != 0) {
			$self->pauseMessage("跑板边计算面积","计算铜面积出错，可能是资料过大内存不足。请保存资料,关闭 GENESIS 重新跑板边");
		}

		open(MYFILE,"$info_file") or die "$!";
		while (<MYFILE>) {
			next if /^\s*$/;
			if(/1st layer exposed/){
				my ($total,$percent)= $_=~/1st layer exposed: (\d.*) sq\/mm \((\d+)%\)/;
				push @masks,["tl",$total,$percent];
			}
			if(/2nd layer exposed/){
				my ($total,$percent)= $_=~/2nd layer exposed: (\d.*) sq\/mm \((\d+)%\)/;
				push @masks,["bl",$total,$percent];
			}
		}
	}


	#print Dumper \@inners;
	#print Dumper \@outs;
	#print Dumper \@masks;

	return \@outs,\@inners,\@masks;
}


#---获取set ok个数尺寸
sub get_set_ok_count{
	my $self = shift;
	my $step = shift;

	my %ha;
	$self->INFO(entity_type=>"step",
			 entity_path=>"$self->{Job}/$step",
			 data_type  =>"REPEAT",
	         parameters => "step");
	my @allstep = @{$self->{doinfo}{gREPEATstep}};
	foreach my $one (@allstep) {
		if ($one =~ /^set$/) {
			$ha{set}{count}++;
			if ($ha{set}{count}==1) {
				my ($xmin,$ymin,$xmax,$ymax) =  $self->step_PROF_LIMITS("set","mm");	
				my $wide = sprintf ("%0.2f",$xmax-$xmin);
				my $long = sprintf ("%0.2f",$ymax-$ymin);
				$ha{set}{size} = $wide.' X '.$long;

				$self->INFO(entity_type=>"step",
						 entity_path=>"$self->{Job}/set",
						 data_type  =>"REPEAT",
						 parameters => "step");
				my @all = @{$self->{doinfo}{gREPEATstep}};
				foreach my $unit (@all) {
					if ($unit =~ /^ok$/) {
						$ha{ok}{count}++;
						if ($ha{ok}{count}==1) {
							my ($xmin,$ymin,$xmax,$ymax) =  $self->step_PROF_LIMITS("ok","mm");	
							my $wide = sprintf ("%0.2f",$xmax-$xmin);
							my $long = sprintf ("%0.2f",$ymax-$ymin);
							$ha{ok}{size} = $wide.' X '.$long;
						}
					}
				}

			}
		}
		if ($one =~ /^ok$/) {
			$ha{ok}{count}++;
			if ($ha{ok}{count}==1) {
				my ($xmin,$ymin,$xmax,$ymax) =  $self->step_PROF_LIMITS("ok","mm");	
				my $wide = sprintf ("%0.2f",$xmax-$xmin);
				my $long = sprintf ("%0.2f",$ymax-$ymin);
				$ha{ok}{size} = $wide.' X '.$long;
			}
		}
	}
	return \%ha;
}


#---强达板边锣边尺寸
sub get_user_job_panel_info{
	my $self         = shift;
	my $jobInfoFile  = shift;
	#my $jobInfo = "$self->{JOB_PATH}/user/JOB_values";
	if (-e $jobInfoFile) {
		my $lines = $self->ReadFile($jobInfoFile);
		foreach my $line  (@{$lines}) {
			my @all = split /\;/,$line;
			foreach my $one (@all) {
				if ($one =~ /^set/) {
					$one =~ s/^set//;
					$one =~ s/^\s+//;

					my $long = length($one);
					my $i = 0;
					while ($i < $long) {
						my $char = substr($one,$i,1);
						if ($char =~ /\s/) {
							last;
						}
						$i++;
					}	
					
					my $key = substr($one,0,$i);
					my $value = substr($one,$i,1000);
					$self->{jobRunPnlInfo}{$key} = $value;
				}
			}
		}
	}
	#print Dumper $self->{jobRunPnlInfo};
	#---层结构
	#$self->get_stackups();
}



sub get_user_job_info{
	my $self = shift;



	#---不存在$self->{JOB_PATH}/user/mes_paramter_info这个文件时,设置默认值
	$self->{jobUserInfo}{'板厚'} = 0;
	$self->{jobUserInfo}{'外层铜厚'} = '0/0';
	$self->{jobUserInfo}{'外层线路工艺'} = "正片";








	my $jobInfo = "$self->{JOB_PATH}/user/mes_paramter_info";
	if (-e $jobInfo) {
		my $lines = $self->ReadFile($jobInfo);
		open (MY_LOG_FILE,"<:encoding(utf-8)",$jobInfo) or die "$!";
	    while (<MY_LOG_FILE>) {
		     chomp $_;
			 my $line = $_;
			my @all = split /\==/,$line;
			$self->{jobUserInfo}{$all[0]} = $all[1];

	   	 }
    	close(MY_LOG_FILE);
	}
	else{
		$self->tipMessage("提示","未取到MES阻焊颜色,字符颜色,层数,客户型号,板厚,外层线路工艺,阻焊油墨型号,铜厚等信息,请检查",500,300);
	}



  
	#--初始化 rick 2022/12/2
	$self->{jobRunPnlInfo} = {};

	#--旧跑板边的锣后边值 JOB_values 到 $self->{jobRunPnlInfo} rick 2022/12/2
    $self->get_user_job_panel_info("$self->{JOB_PATH}/user/JOB_values");

	#--新跑板边的锣边值 JOB_values1  rick 2022/12/2, 用这个覆盖旧跑板的
    #$self->get_user_job_panel_info("$self->{JOB_PATH}/user/JOB_values1");

	#print Dumper $self->{jobUserInfo};
	#---层结构
	$self->get_stackups();
}



sub get_stackups{
	my $self = shift;

	my @allInnerLays = ();
    foreach my $layer (sort{$self->{matrix}{$a}{row}<=>$self->{matrix}{$b}{row}} keys %{$self->{matrix}} ) {
		if ( $self->{matrix}{$layer}{context} =~ /board/i && $self->{matrix}{$layer}{side} =~/inner/){
			push @allInnerLays,$layer;
		}
		if ($layer =~ /^(\d{1,2})\-(\d{1,2})fz$/) { 
			push @allInnerLays,$layer;
		}
	}

	#---没有这个文件时 "$self->{JOB_PATH}/user/mes_staup_info 设置默认值
	foreach my $layer (sort{$self->{matrix}{$a}{row}<=>$self->{matrix}{$b}{row}} keys %{$self->{matrix}} ) {
		$self->{matrix}{$layer}{another} = '';
		$self->{matrix}{$layer}{xinthk} = '';
		$self->{matrix}{$layer}{cuthk} = '';
		$self->{matrix}{$layer}{gongyi} = '';
		$self->{matrix}{$layer}{xscale}  = 0;
		$self->{matrix}{$layer}{yscale}  = 0;

		if ( $self->{matrix}{$layer}{context} =~ /board/i && $self->{matrix}{$layer}{side} =~/inner/){
			$self->{matrix}{$layer}{gongyi} = "负片";
			$self->{matrix}{$layer}{cuthk} = '0/0';
			$self->{matrix}{$layer}{xinthk} = '0';
		}
		if ($layer =~ /^tl$/) {
			$self->{matrix}{$layer}{another} = "bl";
			$self->{matrix}{$layer}{xinthk} = $self->{jobUserInfo}{'板厚'};
			$self->{matrix}{$layer}{cuthk} = $self->{jobUserInfo}{'外层铜厚'};
			$self->{matrix}{$layer}{gongyi}  = $self->{jobUserInfo}{'外层线路工艺'};
		}
		if ($layer =~ /^bl$/) {
			$self->{matrix}{$layer}{another} = "tl";
			$self->{matrix}{$layer}{xinthk} = $self->{jobUserInfo}{'板厚'};
			$self->{matrix}{$layer}{cuthk} = $self->{jobUserInfo}{'外层铜厚'};
			$self->{matrix}{$layer}{gongyi}  = $self->{jobUserInfo}{'外层线路工艺'};
		}

		if ($layer =~ /^(\d{1,2})\-(\d{1,2})fz$/) { 
			my ($num1,$num2) = ($1,$2);
			if ($num1<$num2) {
				my $another = "l".$num2."b";
				if (exists $self->{matrix}{$another}) {#--获取对应的 BOTTOM 面
					$self->{matrix}{$layer}{another}   = $another;
					$self->{matrix}{$another}{another} = $layer;
				}
				#$self->PAUSE("$layer ==bot=$another  =$self->{matrix}{$layer}{another}");
			}elsif ($num1>$num2) {
				my $another = "l".$num2."t";
				if (exists $self->{matrix}{$another}) {#--获取对应的 TOP 面
					$self->{matrix}{$layer}{another}   = $another;
					$self->{matrix}{$another}{another} = $layer;
				}
				#$self->PAUSE("$layer ==top=$another  =$self->{matrix}{$layer}{another}");
			}

		}
		if ($layer =~ /^l(\d{1,2})t$/) {
			my $num2 = $1;
			my $num1 = $num2+1;
			my $normLayer = "l".$num1."b";
			my $another = $num1."-".$num2."fz";
			
			if ( ! exists $self->{matrix}{$normLayer} &&  exists $self->{matrix}{$another}) {#--获取对应的 BOTTOM 面
				$self->{matrix}{$layer}{another} = $another;
				$self->{matrix}{$another}{another} = $layer;
			}elsif (exists $self->{matrix}{$normLayer}){#--获取对应的 BOTTOM 面
				$self->{matrix}{$layer}{another} = $normLayer;
			}
			#$self->PAUSE("$layer ===$another  =$self->{matrix}{$layer}{another}");

		}
		if ($layer =~ /^l(\d{1,2})b$/) {
			my $num2 = $1;
			my $num1 = $num2-1;
			my $normLayer = "l".$num1."t";
			my $another = $num1."-".$num2."fz";
			
			if ( ! exists $self->{matrix}{$normLayer} &&  exists $self->{matrix}{$another}) {#--获取对应的 TOP 面
				$self->{matrix}{$layer}{another} = $another;
				$self->{matrix}{$another}{another} = $layer;
			}elsif (exists $self->{matrix}{$normLayer}){#--获取对应的 TOP 面
				$self->{matrix}{$layer}{another} = $normLayer;
			}
			#$self->PAUSE("$layer ===$another  =$self->{matrix}{$layer}{another}");

		}

	}

	for (my $i = 0; $i <= $#allInnerLays; $i++) {#--rick 2023/12/19 根据相邻层来判断 Core
		if ($i % 2 == 0) {#--top
			my $topLay = $allInnerLays[$i];
			my $botLay = $allInnerLays[$i+1];
			$self->{matrix}{$topLay}{another} = $botLay;
			$self->{matrix}{$botLay}{another} = $topLay;
		}
	}

    #---存在这个文件时,重新设置上面的值
	my $jobInfo = "$self->{JOB_PATH}/user/mes_staup_info";
	if (-e $jobInfo) {
		my %ha;	
		my @ppMaterial = ();
		my @xinMaterial = ();
		my $lines = $self->ReadFile($jobInfo);
		for my $i (0..$#{$lines}) {
			if ($lines->[$i] =~ /\;1=\;2=Core\;3=(\S+)\;4=(\S+)?\;/) {
				my $xinThk = $1;
				my $xinMaterial = $2;
				my ($top,$topcuthk,$bot,$botcuthk);
				for (my $m=$i-1;$m>=0;$m--) {
					if ($lines->[$m] =~ /\;1=(\S+)\;2=OZ\;3=(\S+)\;4=(\S+)?\;/) {
						($top,$topcuthk) = ($1,$2);
						$top = ($m==0) ? "tl" : "l".$top."t";
						last;
					}
				}
				for  (my $n=$i+1;$n<=$#{$lines};$n++) {
					if ($lines->[$n] =~ /\;1=(\S+)\;2=OZ\;3=(\S+)\;4=(\S+)?\;/) {
						($bot,$botcuthk) = ($1,$2);
						my $num = $bot;
						my $allNumEmsStackLin = $#{$lines} - 1;
						$bot = ($n==$allNumEmsStackLin) ? "bl" : "l".$bot."b";

						#$self->PAUSE("top:$top==bot:$bot  $n==$allNumEmsStackLin ==$lines->[$n]");

						if (! exists $self->{matrix}{$bot}) {
							#my $numpre = $num+1;
							my $numpre = $num;
							$num --;
							$bot = "$numpre"."-".$num."fz";
						}
						
						last;
					}
				}
				if ($top =~ /tl/) {#--rick 2023/12/15 检测外层 top 是否有辅助菲林
					if (exists $self->{matrix}{$bot}{another}) {
						if ($self->{matrix}{$bot}{another} =~ /fz/i) {
							$top = $self->{matrix}{$bot}{another};
						}
					}
				}elsif ($bot =~ /bl/) {#--rick 2023/12/15 检测外层 bottom 是否有辅助菲林
					if (exists $self->{matrix}{$top}{another}) {
						if ($self->{matrix}{$top}{another} =~ /fz/i) {
							$bot = $self->{matrix}{$top}{another};
						}
					}
				}
				$ha{$top}{another} = $bot;
				$ha{$top}{xinthk} = $xinThk;
				$ha{$top}{cuthk} = $topcuthk;
				$ha{$bot}{another}  = $top;
				$ha{$bot}{xinthk} = $xinThk;
				$ha{$bot}{cuthk} = $botcuthk;
				push @{$self->{allXinthk}}, [$xinThk,$top,$bot];

				#$self->PAUSE("$top= $ha{$top}{another} $ha{$top}{xinthk} $ha{$top}{cuthk}===$bot =$ha{$bot}{another} $ha{$bot}{xinthk}  $ha{$bot}{cuthk}");

				@{$self->{allXinthk}} = sort {$a->[0]<=>$b->[0]} @{$self->{allXinthk}};

			}
		}
		


		if (%ha) {
			foreach my $layer (sort{$self->{matrix}{$a}{row}<=>$self->{matrix}{$b}{row}} keys %{$self->{matrix}} ) {
				if ( $self->{matrix}{$layer}{context} =~ /board/i && $self->{matrix}{$layer}{side} =~/inner/ || $layer =~ /^(\d{1,2})\-(\d{1,2})fz$/){
					if (exists $ha{$layer}) {
						#$self->{matrix}{$layer}{another} = $ha{$layer}{another};
						$self->{matrix}{$layer}{xinthk}  = $ha{$layer}{xinthk};
						$self->{matrix}{$layer}{cuthk}   = $ha{$layer}{cuthk};
						$self->{matrix}{$layer}{gongyi}  = "负片";
						#my $anotherLay = $self->{matrix}{$layer}{another};
						#$self->PAUSE("$layer === $self->{matrix}{$layer}{another} $self->{matrix}{$layer}{xinthk}  $self->{matrix}{$layer}{cuthk}");
						#$self->PAUSE("$layer === $self->{matrix}{$layer}{another}");
					}
				}
				if ($layer =~ /^tl$/) {
					$self->{matrix}{$layer}{another} = "bl";
					$self->{matrix}{$layer}{xinthk} = $self->{jobUserInfo}{'板厚'};
					$self->{matrix}{$layer}{cuthk} = $self->{jobUserInfo}{'外层铜厚'};
					$self->{matrix}{$layer}{gongyi}  = $self->{jobUserInfo}{'外层线路工艺'};
				}
				if ($layer =~ /^bl$/) {
					$self->{matrix}{$layer}{another} = "tl";
					$self->{matrix}{$layer}{xinthk} = $self->{jobUserInfo}{'板厚'};
					$self->{matrix}{$layer}{cuthk} = $self->{jobUserInfo}{'外层铜厚'};
					$self->{matrix}{$layer}{gongyi}  = $self->{jobUserInfo}{'外层线路工艺'};
				}

			}			
		}

		#--芯板与PP材料
		for my $i(0..$#{$lines}) {
			if ($lines->[$i] =~ /\;1=\;2=Core\;3=(\S+)\;4=(\S+)\;/) {
				my $material = $2;
				$material =~ s/\-//g;
				if (! grep (/^$material$/,@xinMaterial)) { 
					push @xinMaterial,$material;
				}

			}

			if ($lines->[$i] =~ /\;1=\;2=PP\;3=(\S+)\;4=(\S+)?\;/) {
				my $material = $1;
				my @all = split /\+/,$material;
				@all = map {my $data = $_; $data =~ s/\(\S+\)//g; $_ = $data;} @all;  #--多张P片
				foreach my $one (@all) {
					if (! grep (/^$one$/,@ppMaterial)) { 
						push @ppMaterial,$one;
					}
				}								
			}
		}

		$self->{ppMaterial} = join ";", @ppMaterial;
		$self->{xinMaterial} = join ";", @xinMaterial;

		$self->set_xyscale_values;

	}
	else{
		$self->tipMessage("提示","未取到MES层压结构信息,请检查!",500,300);
	}
}


	
				

													






#===================================================================
#===================================================================
1;

