#!/usr/bin/perl

=head
     01/29/16 HongBaiQing
=cut

package OutputCheck;

use strict;
use warnings;
use Encode;
use File::Basename;
use File::Copy;
use File::Path qw(make_path);
use utf8;
use Data::Dumper;
#=====================================================

#=========================================================
#=====================检查部分===============================
#----检查是否有大小一样的重叠孔
sub check_drill_same_positon{

	my $self = shift;
	my $step = shift;
	my $drill = shift;


	my %ha;
	my $filedrl = $self->FEATURES_break_sr($step,$drill,"mm");
	my $drlP = $self->ReadFile($filedrl);
	unlink $filedrl;
	foreach my $one (@{$drlP}) {
		if ($one =~ /^#P/i) {
			my ($x,$y,$size) = (split /\s+/,$one)[1,2,3];
			$size =~ s/^r//g;
			my $str = $size.'&'.$x.'&'.$y;
			$ha{$str}++;
		}
	}

	my @all = ();
	foreach my $one (keys %ha) {
		if ($ha{$one}>1) {
			my $size = (split '&',$one)[0];
			if (! grep /^$size$/,@all) {
				push @all,$size;
			}
		}
	}

	if (@all) {
		@all = map {my $data = $_; $data = "r$data"; $_ = $data;} @all;
		my $str =  join ',',@all;
		$self->pauseMessage("错误","$drill 层刀径$str 的孔有重叠,请检查",500,300);
		$self->PAUSE("Check!");
		$self->open_step($step,"inch","no");

	}





}



#=========================================================
#---检查PANEL边钻孔层型号孔是否和厂编一致
sub check_job_num{
	my $self = shift;
	if ($self->{Step} !~ /panel/) {
		return;
	}

	foreach my $layer (sort{$self->{list}{$a}{row}<=>$self->{list}{$b}{row}} keys %{$self->{list}} ) {
		if ($layer =~ /^drl/) {
			my $tmpFile  = $self->FEATURES($self->{Step},$layer);
			my $features = $self->ReadFile($tmpFile);
			unlink $tmpFile;
			foreach my $line (@{$features}) {
				if ($line =~ /canned/ && $line !~ /\*/) {					
					my $text = (split /\s/,$line)[10];
					if ($text !~ /\d+/) {
						next;
					}

					$text =~ s/\'//g;
					if ($self->{Job} !~ /$text/) {
						$self->pauseMessage("错误","型号孔与厂编不一致 $text ",500,300);
						$self->PAUSE("Check!");
						$self->units("inch");
					}
				}
			}
		}
	}
}

#----检查有没有物件进单元
sub check_more_feature{
	my $self = shift;

	if ($self->{Step} !~ /panel/) {
		return;
	}


	my $cuLayer = "cu$$";
	$self->create_layer($cuLayer);
	$self->affected_one_layer($cuLayer); 
	$self->add_surf($self->{PROF_xmin},$self->{PROF_ymin},$self->{PROF_xmax},$self->{PROF_ymax},"positive","no");
	$self->COM("fill_params,type=solid,origin_type=datum,solid_type=surface,\
				std_type=line,min_brush=1,use_arcs=yes,symbol=,dx=0.1,dy=0.1,\
				x_off=0,y_off=0,std_angle=45,std_line_width=10,std_step_dist=50,\
				std_indent=odd,break_partial=yes,cut_prims=no,outline_draw=no,\
				outline_width=0,outline_invert=no");
	$self->COM("sr_fill,polarity=negative,step_margin_x=0,step_margin_y=0,\
					step_max_dist_x=100,step_max_dist_y=100,sr_margin_x=-0.0002,sr_margin_y=-0.0002,\
					sr_max_dist_x=0,sr_max_dist_y=0,nest_sr=no,consider_feat=no,consider_drill=no,\
					consider_rout=no,dest=affected_layers,attributes=no");
	$self->sel_contourize;


	foreach my $layer (sort{$self->{list}{$a}{row}<=>$self->{list}{$b}{row}} keys %{$self->{list}} ) {
		if ($self->{list}{$layer}{new_type} =~ /pdf_part/) {
			next;
		}
		$self->work_affected_layer($layer); 
		$self->sel_ref_feat($cuLayer,"touch");
		if ($self->COM("get_select_count")) {
			$self->PAUSE("Some features in unit,Check");
			$self->units("inch");
		}
	}

	$self->delete_layer($cuLayer);
}


#==============
#----钻孔输出先检查SLOT是否有ROUT_CHAIN属性
sub check_rout_chain{
	my $self = shift;
	if ($self->{Step} !~ /panel/) {
		return;
	}

	foreach my $layer (sort{$self->{list}{$a}{row}<=>$self->{list}{$b}{row}} keys %{$self->{list}} ) {
		if ($layer =~ /^drl|^2nd/) {
			my $tmpFile  = $self->FEATURES_break_sr($self->{Step} ,$layer,'inch');
			my $features = $self->ReadFile($tmpFile);
			unlink $tmpFile;
			if ( grep (/rout_chain/,@{$features})) { 
				$self->PAUSE("$layer have rout_chain,Check");
				$self->units("inch");
			}
		}
	}	
}




#----检查拼板间距
sub check_sr_dis{
	my $self = shift;
	my $outName = shift;

	if ($self->{Step} !~ /panel/) {
		return;
	}
	$self->open_step($self->{Step},"inch","yes");
	my $cuLayer = "cu$$";
	$self->create_layer($cuLayer);
	$self->affected_one_layer($cuLayer); 

	$self->add_surf($self->{PROF_xmin},$self->{PROF_ymin},$self->{PROF_xmax},$self->{PROF_ymax},"positive","no");
	$self->COM("fill_params,type=solid,origin_type=datum,solid_type=surface,\
				std_type=line,min_brush=1,use_arcs=yes,symbol=,dx=0.1,dy=0.1,\
				x_off=0,y_off=0,std_angle=45,std_line_width=10,std_step_dist=50,\
				std_indent=odd,break_partial=yes,cut_prims=no,outline_draw=no,\
				outline_width=0,outline_invert=no");
	$self->COM("sr_fill,polarity=negative,step_margin_x=0,step_margin_y=0,\
					step_max_dist_x=100,step_max_dist_y=100,sr_margin_x=-0.0002,sr_margin_y=-0.0002,\
					sr_max_dist_x=0,sr_max_dist_y=0,nest_sr=no,consider_feat=no,consider_drill=no,\
					consider_rout=no,dest=affected_layers,attributes=no");
	$self->sel_contourize;
	$self->COM("fill_params,type=solid,origin_type=datum,solid_type=fill,\
			std_type=line,min_brush=10,use_arcs=yes,symbol=,dx=0.1,dy=0.1,\
			x_off=0,y_off=0,std_angle=45,std_line_width=10,std_step_dist=50,\
			std_indent=odd,break_partial=yes,cut_prims=no,outline_draw=no,\
			outline_width=0,outline_invert=no");
	$self->COM("sel_fill");
	$self->sel_contourize;
	$self->sel_resize(0.0003);
	$self->filter_reset;
	$self->filter_area_strt();
	my $count1 = $self->COM("get_select_count");
	my $size = 1498/25.4;
	$self->sel_contourize($size);
	$self->sel_resize($size);
	$self->sel_contourize;
	$self->filter_area_strt();
	my $count2 = $self->COM("get_select_count");

	if ($count2 < $count1) {
		my $dir = dirname $outName;
		my $file = "$dir/拼板间距小于1.5.txt";
		$file = encode('gb2312',$file);
		$self->WriteFile($file,"\n");
	}
	$self->delete_layer($cuLayer);
}


#---检查nv与drl是不是同一圆心
sub check_nv_drl{
	my $self = shift;
	my $nv = 0;
	my $step = "edit";
	if ( $self->step_EXISTS($step)  =~ /yes/i)  {
		if (exists $self->{matrix}{drl} && exists $self->{matrix}{nv}) {
			my %drl;
			my $filenv = $self->FEATURES_break_sr($step,"nv");
			my $nvP = $self->ReadFile($filenv);
			my $filedrl = $self->FEATURES_break_sr($step,"drl");
			my $drlP = $self->ReadFile($filedrl);
			unlink $filenv;
			unlink $filedrl;

			foreach my $one (@{$drlP}) {
				if ($one =~ /^#P/i) {
					my ($x,$y) = (split /\s+/,$one)[1,2];
					$x = sprintf ("%0.4f",$x);
					$y = sprintf ("%0.4f",$y);
					my $key = "${x}$y";
					$drl{$key}++;
				}
			}
			foreach my $one (@{$nvP}) {
				if ($one =~ /^#P/i) {
					my ($x,$y) = (split /\s+/,$one)[1,2];
					$x = sprintf ("%0.4f",$x);
					$y = sprintf ("%0.4f",$y);
					my $key = "${x}$y";
					if (! exists $drl{$key}) {
						 $nv=1;
					}
				}
			}
		}
	}

	if ($nv) {
		$self->pauseMessage("提示","nv与drl有钻孔不同圆心",500,300);
		$self->PAUSE("Check !");
	}
}

#----检查有没有重孔
sub check_dup_hole{
	my $self = shift;
	my $layer = shift;
	my $dup = 0;
	#----当有set 在set检查，没有则在edit检查
	foreach my $step ("set","edit") {
		if ( $self->step_EXISTS($step)  =~ /yes/i)  {
			my %drl;
			my $filedrl = $self->FEATURES_break_sr($step,$layer);
			my $drlP = $self->ReadFile($filedrl);
			unlink $filedrl;
			foreach my $one (@{$drlP}) {
				if ($one =~ /^#P/i) {
					my ($x,$y,$size) = (split /\s+/,$one)[1,2,3];
					$x = sprintf ("%0.4f",$x);
					$y = sprintf ("%0.4f",$y);
					my $key = "${x}${y}$size";
					$drl{$key}++;
				}
			}
			foreach my $key (keys %drl) {
				if ($drl{$key} > 1) {
					$dup = 1;
					last;
				}
			}
			last;
		}
	}
	
	if ($dup) {
		$self->pauseMessage("提示","$layer 层有叠孔",500,300);
		$self->PAUSE("Check !");
	}
}

#--有没有大4.05的孔没有加预钻孔 4.0mm的有没加引孔
sub check_yuzuan_hole{
	my $self = shift;
	my $layer = shift;
	my $yuzuan = 0;
	my $yihole = 0;
	my $slot = 0;  
	my $slot2 = 0;
	my $noslotyi= 0;
	

	
	#----当有set 在set检查，没有则在edit检查
	foreach my $step ("set","edit") {
		if ( $self->step_EXISTS($step)  =~ /yes/i)  {
			my %drl;
			my %noSlotYu;
			my $tmpFile = $ENV{GENESIS_TMP}.'/info'.$$;
			$self->COM("info,out_file=$tmpFile,units=mm,args= -t layer -e $self->{Job}/$step/$layer -m display -d FEATURES -o break_sr+feat_index");
			my $drlP = $self->ReadFile($tmpFile);
			unlink $tmpFile;

			foreach my $one (@{$drlP}) {
				if ($one =~ /#P/i) {
					my ($id,$x,$y,$size) = (split /\s+/,$one)[0,2,3,4];
					$id =~ s/^#//g;
					$size =~ s/^r//g;
					$drl{$id}{x} =sprintf ("%0.4f",$x);
					$drl{$id}{y} =sprintf ("%0.4f",$y);
					$drl{$id}{size} =sprintf ("%0.4f",$size);
				}
			}

			 #---检查小于2D的条孔的尾数是不是5 6
			foreach my $one (@{$drlP}) {
				if ($one =~ /#L/i) {
					my ($id,$x1,$y1,$x2,$y2,$size) = (split /\s+/,$one)[0,2,3,4,5,6];
					$id =~ s/^#//g;
					$size =~ s/^r//g;
					$size = $size/1000;
					my $long = sqrt(($x1-$x2)**2+($y1-$y2)**2);
					if ($long<$size && $size !~ /5$|6$/) {
						$slot = 1;                            #--小于2D的加尾数不对
					}

					if ($size !~ /\d[1-9]$/) { #--没有加尾孔
						$slot2 = 1;
					}

					if ($long < $size) {  #--小于2D的没加预钻孔
						$noSlotYu{$id}=0;
						foreach my $id2 (keys %drl) {						
							my $dis1 = sqrt( ($x1-$drl{$id2}{x})**2+($y1-$drl{$id2}{y})**2 );
							my $dis2 = sqrt( ($x2-$drl{$id2}{x})**2+($y2-$drl{$id2}{y})**2 );							
							if ($dis1<$size/2 || $dis2<$size/2) {
								$noSlotYu{$id} = 1;
								last;
							}							
						}
					}
				}
			}

			if ($slot) {
				$self->pauseMessage("提示","$layer 有小于2D的条孔的尾数不是5,6",500,300);
				$self->PAUSE("Check !");
			}

			if ($slot2) {
				$self->pauseMessage("提示","$layer 槽孔未加定义尾数",500,300);
				$self->PAUSE("Check !");
			}



			foreach my $id (keys %noSlotYu) {
				if ($noSlotYu{$id} == 0) {
					$noslotyi = 1;
				}
			}
			if ($noslotyi) {
				$self->pauseMessage("提示","$layer 短槽孔未加预钻孔",500,300);
				$self->PAUSE("Check !");
			}


	

			foreach my $id (keys %drl) {
				#---检查有没有加预钻孔
				if ($drl{$id}{size}>=4050) {
					foreach my $id2 (keys %drl) {
						if ($id2 == $id) {
							next;
						}						
						my $dis = sqrt( ($drl{$id}{x}-$drl{$id2}{x})**2+($drl{$id}{y}-$drl{$id2}{y})**2 );
						if ($dis<2) {
							$drl{$id}{yuzuan} = "yes";
						}
					}
				}
				#---检查有没有加引孔
				elsif ($drl{$id}{size}>=4000) {
					foreach my $id2 (keys %drl) {
						if ($id2 == $id) {
							next;
						}						
						my $dis = sqrt( ($drl{$id}{x}-$drl{$id2}{x})**2+($drl{$id}{y}-$drl{$id2}{y})**2 );
						if ($dis<2) {
							$drl{$id}{yi} = "yes";
						}
					}
				}
			}


			foreach my $id (keys %drl) {
				if ($drl{$id}{size}>=4050 && ! exists $drl{$id}{yuzuan}) {
					$yuzuan = 1;	
				}

				if ($drl{$id}{size}>=4000 && $drl{$id}{size}<4050 && ! exists $drl{$id}{yi}) {
					$yihole= "1";	
				}
			}

			if ($yuzuan) {
				$self->pauseMessage("提示","$layer 有>=4.050mm的也没加预钻孔",500,300);
				$self->PAUSE("Check !");
			}

			if ($yihole) {
				$self->pauseMessage("提示","$layer 有>=4.00mm的也没加引孔",500,300);
				$self->PAUSE("Check !");
			}
			last;
		}
	}
}


1;

